/* GLMorph.h created by epeyton on Thu 03-Dec-1998 */

#import <AppKit/AppKit.h>
#import <ScreenSaver/ScreenSaver.h>
#import <OpenGL/gl.h>
#import <OpenGL/glu.h>

#define EVENT_HORIZON_GRAV .5

#define ESCAPE_HORIZON 30 
#define ESCAPE_HORIZON_SQR (ESCAPE_HORIZON * ESCAPE_HORIZON)


// ------
// Frames per second (FPS) statistic variables and routine.

#define FRAME_RATE_SAMPLES 50


// Structure to hold all the data for each particle.
typedef struct {
   int Running;
   float Pos[3];  // Position.
   float Vel[3];  // Velocity.
   float Grav[3]; // Acceleration.
   float Color[3];
} Particle;

// "Name" for first (and only) OpenGL display list.
#define STAR_FIELD 1




@interface BlackHole : ScreenSaverView
{        
    NSOpenGLView *_view;
    BOOL _initedGL;
    int Texture_ID;
    int Window_ID;
    
    // We'll request a sphere from the GLU library at run-time.
    struct GLUquadric *Black_Hole;

    
    // Pointer for allocated array of Particles.
    Particle *Parts;
    
    // Particle status variables.
    int Parts_Running;
    int Parts_Allocated;
    int Parts_LastUnused;
    
    // Number of parts initially in the system.  Make sure there's at least
    // one over a hundred (101, 801), so our Root particle doesn't get deleted.
    int Parts_Num;
    
    float Parts_Brightness;
    
    
    // Drawing flags.
    int Draw_Axis;
    int Draw_Vectors;
    int Draw_Stars;
    int Heads_Up;
    int Texture_On;
    
    
    // Particle Gun variables.
    float Gun_PX;
    float Gun_PY;
    float Gun_PZ;
    
    float Gun_VX;
    float Gun_VY;
    float Gun_VZ;
    float Gun_R;
   float Gun_OffR;
    
    // Backwards firing and off-target probabilities.
    float Gun_Back, Gun_Off;
    
    // '.' key toggels between keypad adjusting gun position and eject vect.
    int Gun_Control;
    
    
    // Orbit and motion settings.
    int On_Orbit;
    int Move_Enable;
    int Move_Step;
    
    // Observer initial placement.
    float Obs_Angle;
    float Obs_Height;
    float Obs_Dist;
    
    // Calculated observer location.
    float Obs[3];
    
    // How quickly do the orbits decay? 
    // Lower number (limit 1) is faster decay.
    
    int Decay_Factor;
    
    // The force of gravity exterted by the black hole.
    float Grav;
    
    // Somewhat arbitrary values for Event and Escape horizons.
    
    int FrameCount;
    float FrameRate;
    
}

@end


