#import "GLConnectivity.h"


@implementation GLConnectivity
    

- (id)initWithFrame:(NSRect)frameRect isPreview:(BOOL)preview
{

    self = [super initWithFrame:frameRect isPreview:preview];
    if (self) {
	NSOpenGLPixelFormatAttribute attribs[] =
        {	NSOpenGLPFAAccelerated,
		NSOpenGLPFADepthSize, 16,
		NSOpenGLPFAMinimumPolicy,
		NSOpenGLPFAClosestPolicy,
                0
        };
	
	NSOpenGLPixelFormat *format = [[[NSOpenGLPixelFormat alloc] initWithAttributes:attribs] autorelease];
		
        _view = [[[NSOpenGLView alloc] initWithFrame:NSZeroRect pixelFormat:format] autorelease];
        [self addSubview:_view];
    }
    

    return self;
}

- (void)dealloc
{
    [_view removeFromSuperview];
    [super dealloc];
}

- (void)setFrameSize:(NSSize)newSize
{
    [super setFrameSize:newSize];
    [_view setFrameSize:newSize];
    _initedGL = NO;

}

- (void)stopAnimation
{
    _initedGL = NO;
    [super stopAnimation];

    return;
}


- (void)startAnimation
{
        [[_view openGLContext] makeCurrentContext];
	
	if (!_initedGL) {
        
	    rBounds3D.left = 0;
	    rBounds3D.right = (int)[self frame].size.width;
	    rBounds3D.top = 0;
	    rBounds3D.bottom = (int)[self frame].size.height;
	    rBounds3D.front = 0;
	    rBounds3D.back = ((int)[self frame].size.width + (int)[self frame].size.height) / 2;
	    
		    glShadeModel(GL_SMOOTH);
	    glEnable( GL_LINE_SMOOTH );
	    glViewport (0, 0, (GLsizei) [self frame].size.width, (GLsizei) [self frame].size.height);
	
	    glMatrixMode(GL_PROJECTION);
	    glLoadIdentity();	
	    //gluPerspective(20.0, (GLfloat) [self frame].size.width / (GLfloat) [self frame].size.height, 01, 100.0);
            gluPerspective(20.0f,(GLfloat)[self frame].size.width/(GLfloat)[self frame].size.height,1.0f,100.0f);
	    
            glMatrixMode(GL_MODELVIEW);
	    glLoadIdentity();
	    
	    minSpeed = kDefaultMin;
	    maxSpeed = kDefaultMax;
	    numPoints = (int)SSRandomFloatBetween(3.0, 40.0);
	    
	    [self InitPoints];
    	    _initedGL = YES;
    
	}
    [super startAnimation];

}

- (void)animateOneFrame
{
        unsigned long i;
        [[_view openGLContext] makeCurrentContext];

	for( i = 0; i < numPoints; i++ )
	{
		[self MovePointRandom: &ourPoints[i] : &ourDirs[i] : &rBounds3D ];
	}
        [self Draw];

    
    return;
}

- (BOOL)hasConfigureSheet { return NO; }
- (NSWindow*)configureSheet { return nil; }

- (void) InitPoints
{
	unsigned long i;
	
	for( i = 0; i < numPoints; i++ )
	{
		ourPoints[i].x = SSRandomFloatBetween(0,boundsWidth);
		ourPoints[i].y = SSRandomFloatBetween(0,boundsHeight);
		ourPoints[i].z = 0;
	

		ourDirs[i].x = (int)SSRandomFloatBetween(minSpeed,maxSpeed);
		if( SSRandomFloatBetween(2.0,3.0) > 2.5 ) ourDirs[i].x = -ourDirs[i].x;
		
		ourDirs[i].y = SSRandomFloatBetween(minSpeed,maxSpeed);
		if( SSRandomFloatBetween(2.0,3.0) > 2.5 ) ourDirs[i].y = -ourDirs[i].y;

		ourDirs[i].z = 0;
		//ourDirs[i].z = Rand( minSpeed, maxSpeed );;
		//if( Rand(2,3) == 3 ) ourDirs[i].z = -ourDirs[i].z;
	}
}

//Draws all the lines & points
- (void) Draw
{    
	unsigned long y, x;

	glClear(GL_COLOR_BUFFER_BIT);
	
	glLoadIdentity();
	glTranslatef( -.5,-.5,-3 );
	
	//Draw all the lines
	glBegin( GL_LINES );
		for( y = 0; y < numPoints; y++ )
		{
			for( x = (y+1); x < numPoints; x++ )
			{
				glColor3f(  colors[ y % kNumColors ].r,
							colors[ y % kNumColors ].g,
							colors[ y % kNumColors ].b );
				glVertex3f( ourPoints[y].x/rBounds3D.right, 
							ourPoints[y].y/rBounds3D.bottom, 
							ourPoints[y].z/rBounds3D.back );

				glColor3f(  colors[ (y+1) % kNumColors ].r,
							colors[ (y+1) % kNumColors ].g,
							colors[ (y+1) % kNumColors ].b );
				glVertex3f( ourPoints[x].x/rBounds3D.right, 
							ourPoints[x].y/rBounds3D.bottom, 
							ourPoints[x].z/rBounds3D.back );
			}
		}
	glEnd();
	
	
	//Draw all the big squares
	glBegin( GL_QUADS );
		for( y = 0; y < numPoints; y++ )
		{
			glColor3f(  colors[ y % kNumColors ].r,
						colors[ y % kNumColors ].g,
						colors[ y % kNumColors ].b );

			glVertex3f( ourPoints[y].x/rBounds3D.right - kSquareSize, 
			ourPoints[y].y/rBounds3D.bottom - kSquareSize, 
			ourPoints[y].z/rBounds3D.back );
			
			glVertex3f( ourPoints[y].x/rBounds3D.right + kSquareSize, 
			ourPoints[y].y/rBounds3D.bottom - kSquareSize, 
			ourPoints[y].z/rBounds3D.back );
			
			glVertex3f( ourPoints[y].x/rBounds3D.right + kSquareSize, 
			ourPoints[y].y/rBounds3D.bottom + kSquareSize, 
			ourPoints[y].z/rBounds3D.back );
			
			glVertex3f( ourPoints[y].x/rBounds3D.right - kSquareSize, 
			ourPoints[y].y/rBounds3D.bottom + kSquareSize, 
			ourPoints[y].z/rBounds3D.back );
		}
	glEnd();

    glFlush();

	
}



//Moves a point, and sends it in a random direction if
//it hits the bound edge. 
- (void) MovePointRandom: (Point3D*) p : (Point3D*) dir: (const Rect3D*) inBounds
{
	p->x += dir->x;
	p->y += dir->y;
	p->z += dir->z;
	
	if( p->x < inBounds->left )
	{
		p->x = inBounds->left+1;
		dir->x = SSRandomFloatBetween(minSpeed, maxSpeed);
	}
	else if( p->x > inBounds->right )
	{
		p->x = inBounds->right-1;
		dir->x = -SSRandomFloatBetween(minSpeed, maxSpeed);
	}

	if( p->y < inBounds->top )
	{
		p->y = inBounds->top+1;
		dir->y = SSRandomFloatBetween(minSpeed, maxSpeed);

	}
	else if( p->y > inBounds->bottom )
	{
		p->y = inBounds->bottom-1;
		dir->y = -SSRandomFloatBetween(minSpeed, maxSpeed);
	}
}

@end

