#import "GLParticles.h"

@implementation GLParticles	

static float Time(void)
{
	UnsignedWide ftime;
	
	Microseconds(&ftime);
	
	return ftime.lo/1000000.0f;
}

static void GetTexture(int resID)
{}

- (void)updateSelections:sender
{   	
    fireworksMode = [typeMatrix selectedRow];
    stereo = [stereoCheck state];
    blueGlasses = [blueGreenCheck state];

    CFPreferencesSetAppValue(@"GLParticlesFWMode",[NSNumber numberWithInt:fireworksMode], @"EPScreenSavers");
    CFPreferencesSetAppValue(@"GLParticlesStereo",[NSNumber numberWithInt:stereo], @"EPScreenSavers");
    CFPreferencesSetAppValue(@"GLParticlesBG",[NSNumber numberWithInt:blueGlasses], @"EPScreenSavers");
    
    CFPreferencesAppSynchronize(@"EPScreenSavers");

}


- (void)DrawParticle:(vec_3) pos :(float) scale
{
	vec_3 delta;
	float xr,yr;
	
	delta.x = camera.x - pos.x;
	delta.y = camera.y - pos.y;
	delta.z = camera.z - pos.z;
	
	yr = radToDeg * atan(delta.x/delta.z);
	xr = radToDeg * atan(delta.y/delta.z);
		
	glPushMatrix();
	
	glTranslatef(pos.x,pos.y,pos.z);
	
	glRotatef(-xr,1,0,0);
	glRotatef(yr,0,1,0);
	
	glScalef(scale,scale,1.0f);
	
	glCallList(1);
	
	glPopMatrix();
}



- (id)initWithFrame:(NSRect)frameRect isPreview:(BOOL)preview
{
    self = [super initWithFrame:frameRect isPreview:preview];
        
    if (self) {
	NSOpenGLPixelFormatAttribute attribs[] =
        {	NSOpenGLPFAAccelerated,
		NSOpenGLPFADepthSize, 16,
		NSOpenGLPFAMinimumPolicy,
		NSOpenGLPFAClosestPolicy,
                0
        };
	
	NSOpenGLPixelFormat *format = [[[NSOpenGLPixelFormat alloc] initWithAttributes:attribs] autorelease];
		
        _view = [[[NSOpenGLView alloc] initWithFrame:NSZeroRect pixelFormat:format] autorelease];
        [self addSubview:_view];
    }
        
    fireworksMode = [(NSNumber *)CFPreferencesCopyAppValue(@"GLParticlesFWMode", @"EPScreenSavers") intValue];
    stereo = [(NSNumber *)CFPreferencesCopyAppValue(@"GLParticlesStereo", @"EPScreenSavers") intValue];
    blueGlasses = [(NSNumber *)CFPreferencesCopyAppValue(@"GLParticlesBG", @"EPScreenSavers") intValue];

        
    return self;
}

- (void)dealloc
{
    [_view removeFromSuperview];
    [super dealloc];
}

- (void)setFrameSize:(NSSize)newSize
{
    [super setFrameSize:newSize];
    [_view setFrameSize:newSize];
    _initedGL = NO;

}


- (void)stopAnimation
{
    _initedGL = NO;
    [super stopAnimation];

    return;
}


- (void)startAnimation
{
    
    [[_view openGLContext] makeCurrentContext];
    if (!_initedGL) {
        int i;
        num_particles = 25;
        texture = 0;
        
        	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	gluPerspective(90.0f,(GLfloat)[self frame].size.width/(GLfloat)[self frame].size.height,0.1f,100.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();


	gluLookAt(camera.x, camera.y, camera.z, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
    
    	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	
	GetTexture(5001);

	glShadeModel(GL_SMOOTH);
	glClearColor(0.0f,0.0f,0.0f,0.0f);
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D,texture);

	if (fireworksMode)
		for (i=0;i<num_particles;i++)
			[self CreateRandomFirework:i];
	else
		for (i=0;i<num_particles;i++)
			[self CreateRandomParticle:i];
	
	glNewList(1,GL_COMPILE);
	
	glBegin(GL_TRIANGLE_STRIP);
		glTexCoord2d(1,1); glVertex3f(1.0f,1.0f,0);
		glTexCoord2d(0,1); glVertex3f(-1.0f,1.0f,0);
		glTexCoord2d(1,0); glVertex3f(1.0f,-1.0f,0);
		glTexCoord2d(0,0); glVertex3f(-1.0f,-1.0f,0);
	glEnd();
	
	glEndList();

        _initedGL = YES;
        
        last = Time();
    }

    [super startAnimation];
}


- (void)animateOneFrame
{
    [[_view openGLContext] makeCurrentContext];
	if (fireworksMode)
		[self drawFireworks];
	else
		[self drawGL];
                glFlush();
                
    return;
}


- (BOOL)hasConfigureSheet { return YES; }
- (NSWindow*)configureSheet { 

    [NSBundle loadNibNamed:@"Particles.nib" owner:self];
    [typeMatrix selectCellAtRow:fireworksMode column:0];
    [stereoCheck setState:stereo];
    [blueGreenCheck setState:blueGlasses];
    return window; 
}

- (void)closeSheet:(id)sender
{
    [NSApp endSheet:window];
}

- (void)drawGL
{
	int i,j;
	float sinyrot,cosyrot,sinxrot,cosxrot;
	float now;
	static float time;
	
	now = Time();
	time = now - last;
	
	yrot += 60.0 * time;
	xrot = 40.0f * cos(6.28 * (float)0.2f * (now-particle[0]->spawnt));
	
	if (stereo)
		yrot -= 1.0;

	
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	sinyrot = sin(yrot*degToRad);
	cosyrot = cos(yrot*degToRad);
	sinxrot = sin(xrot*degToRad);
	cosxrot = cos(xrot*degToRad);
	
	camera.x = 30.0f * sinyrot;
	camera.y = 30.0f * sinxrot;
	camera.z = 30.0f * cosyrot;
        
        glViewport(0,0,[self frame].size.width,[self frame].size.height);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	gluPerspective(90.0f,(GLfloat)[self frame].size.width/(GLfloat)[self frame].size.height,0.1f,100.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();


	gluLookAt(camera.x, camera.y, camera.z, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
	
	glColor4f(1.0f,1.0f,1.0f,0.8f);
	
	for (i=0;i<num_particles;i++)
	{
		vec_3 *p;
		vec_3 old;
		
		int spawned=0;
		
		old.x = particle[i]->pos.x;
		old.y = particle[i]->pos.y;
		old.z = particle[i]->pos.z;
		
		particle[i]->yr+=particle[i]->yv * time;
		
		p = &particle[i]->pos;
		
		p->x = particle[i]->rad * sin(degToRad * particle[i]->yr);
		p->z = particle[i]->rad * cos(degToRad * particle[i]->yr);
		
		p->y = particle[i]->boba * sin(6.28 * particle[i]->bobv * (now-particle[i]->spawnt));
		
		if (stereo)
			glColor4f(1.0, 0.0, 0.0, 0.8);
		else
			glColor4f(particle[i]->r,particle[i]->g,particle[i]->b,0.8f);
		
		[self DrawParticle:*p :4.0f];
		
		for (j=0;j<MAX_CHILDREN;j++)
		{ 
			particle[i]->child[j].life -= time;
			
			if (particle[i]->child[j].life<0)
			{
				particle[i]->child[j].dead = true;
				spawned++;
			}
			else
			{
				particle[i]->child[j].pos.y += particle[i]->gravity.y * time;
			}
			
			if (stereo)
				glColor4f(1.0, 0.0, 0.0, 0.8 * particle[i]->child[j].life / particle[i]->child_maxlife);
			else
				glColor4f(particle[i]->r,particle[i]->g,particle[i]->b,0.8 * particle[i]->child[j].life / particle[i]->child_maxlife);
			
			[self DrawParticle:particle[i]->child[j].pos:1.0f];
		}
		
		old.x -= p->x;
		old.y -= p->y;
		old.z -= p->z;
		
		for (j=0;j<MAX_CHILDREN;j++)
		{
			int totalSpawned = spawned;
			if (particle[i]->child[j].dead)
			{
				spawned--;
				particle[i]->child[j].pos.x = p->x + old.x * (float) spawned/totalSpawned;
				particle[i]->child[j].pos.y = p->y + old.y * (float) spawned/totalSpawned;
				particle[i]->child[j].pos.z = p->z + old.z * (float) spawned/totalSpawned;
				particle[i]->child[j].life = particle[i]->child_maxlife + spawned * particle[i]->child_maxlife/totalSpawned;
				particle[i]->child[j].dead = false;
			}
		}
    }
    
	if (stereo)
	{
    	yrot += 2.0;
    	
    	i = yrot / 360;
		yrot -= 360*i;
		
		sinyrot = sin(yrot*degToRad);
		cosyrot = cos(yrot*degToRad);
		
		camera.x = 30.0f * sinyrot;
		camera.y = 30.0f * sinxrot;
		camera.z = 30.0f * cosyrot;

		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		gluLookAt(camera.x, camera.y, camera.z, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
		
		if (blueGlasses)
			glColor4f(0.0, 0.0, 1.0, 0.8);
		else
			glColor4f(0.0, 1.0, 0.0, 0.8);
		
		for (i=0;i<num_particles;i++)
		{
			vec_3 *p;
			Boolean pdead=false;
			
			p = &particle[i]->pos;
			
			if (particle[i]->child[0].dead)
				[self DrawParticle:*p:3.0f];
			
			if (!particle[i]->child[0].dead)
				for (j=0;j<MAX_CHILDREN;j++)
				{
					[self DrawParticle:particle[i]->child[j].pos:1.0f];
				}
	    }
	    
	    yrot -= 1.0;
    }
    
    last = now;
}

- (void)drawFireworks
{
	int i,j;
	float sinyrot,cosyrot,sinxrot,cosxrot;
	float now;
	float time;
	
	now = Time();
	time = now - last;
	
	yrot += 60.0f * time;
	xrot = 40.0f * cos(6.28 * (float)0.2f * (now-particle[0]->spawnt));
	
	if (stereo)
		yrot -= 1.0;
	
	if (xrot>90)
		xrot = 90;
	else if (xrot<-90)
		xrot = -90;
	
	i = yrot / 360;
	yrot -= 360*i;
		
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	sinyrot = sin(yrot*degToRad);
	cosyrot = cos(yrot*degToRad);
	sinxrot = sin(xrot*degToRad);
	cosxrot = cos(xrot*degToRad);
	
	camera.x = 30.0f * sinyrot;
	camera.y = 30.0f * sinxrot;
	camera.z = 30.0f * cosyrot;

	glViewport(0,0,[self frame].size.width,[self frame].size.height);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	gluPerspective(90.0f,[self frame].size.width/[self frame].size.height,0.1f,100.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(camera.x, camera.y, camera.z, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
	
	for (i=0;i<num_particles;i++)
	{
		vec_3 *p;
		Boolean pdead=false;
		
		p = &particle[i]->pos;
		
		particle[i]->vel.y += particle[i]->gravity.y * time;
		
		p->x += particle[i]->vel.x * time;
		p->y += particle[i]->vel.y * time;
		p->z += particle[i]->vel.z * time;
		
		if (stereo)
			glColor4f(1.0, 0.0, 0.0, 0.8);
		else
			glColor4f(particle[i]->r,particle[i]->g,particle[i]->b,0.8f);
		
		if (particle[i]->child[0].dead)
			[self DrawParticle:*p :3.0f];
		
		if (!particle[i]->child[0].dead)
			for (j=0;j<MAX_CHILDREN;j++)
			{ 
				particle[i]->child[j].life -= time;
				
				if (particle[i]->child[j].life<0)
				{
					particle[i]->child[j].dead = true;
					pdead = true;
				}
				else
				{
					particle[i]->child[j].vel.y += particle[i]->gravity.y * time;
					
					particle[i]->child[j].pos.x += particle[i]->child[j].vel.x * time;
					particle[i]->child[j].pos.y += particle[i]->child[j].vel.y * time;
					particle[i]->child[j].pos.z += particle[i]->child[j].vel.z * time;
				}
				
				if (!stereo)
					glColor4f(particle[i]->r,particle[i]->g,particle[i]->b,0.8f);
				
				[self DrawParticle:particle[i]->child[j].pos :1.0f];
			}

		else if (now - particle[i]->spawnt>2.0f)
			for (j=0;j<MAX_CHILDREN;j++)
			{
				particle[i]->child[j].dead = false;
				particle[i]->child[j].life = particle[i]->child_maxlife;
				particle[i]->child[j].pos.x = particle[i]->pos.x;
				particle[i]->child[j].pos.y = particle[i]->pos.y;
				particle[i]->child[j].pos.z = particle[i]->pos.z;
				particle[i]->child[j].vel.x = (float)Random()/1000.0f;
				particle[i]->child[j].vel.y = (float)Random()/1000.0f;
				particle[i]->child[j].vel.z = (float)Random()/1000.0f;
			}
		
		if (pdead)
			[self CreateRandomFirework:i];
		
    }
    
    if (stereo)
    {
    	yrot += 2.0;
    	
    	i = yrot / 360;
		yrot -= 360*i;
		
		sinyrot = sin(yrot*degToRad);
		cosyrot = cos(yrot*degToRad);
		
		camera.x = 30.0f * sinyrot;
		camera.y = 30.0f * sinxrot;
		camera.z = 30.0f * cosyrot;

		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		gluLookAt(camera.x, camera.y, camera.z, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
		
		if (blueGlasses)
			glColor4f(0.0, 0.0, 1.0, 0.8);
		else
			glColor4f(0.0, 1.0, 0.0, 0.8);
		
		for (i=0;i<num_particles;i++)
		{
			vec_3 *p;
			Boolean pdead=false;
			
			p = &particle[i]->pos;
			
			if (particle[i]->child[0].dead)
				[self DrawParticle:*p :3.0f];
			
			if (!particle[i]->child[0].dead)
				for (j=0;j<MAX_CHILDREN;j++)
				{
					[self DrawParticle:particle[i]->child[j].pos :1.0f];
				}
	    }
	    
	    yrot -= 1.0;
    }
    
    last = now;
}
- (void)CreateRandomParticle:(int) i
{
	int j;
            
	particle[i] = malloc(sizeof(particles));
	
	particle[i]->r= (float)(Random()+32000)/65000.0f;
	particle[i]->g= (float)(Random()+32000)/65000.0f;
	particle[i]->b= (float)(Random()+32000)/65000.0f;
	
	particle[i]->yr=(float)Random()/92.0f;
	particle[i]->yv=(float)Random()/150.0f;
	
	particle[i]->spawnt = Time();
	particle[i]->bobv = (float)Random()/16000.0f;
	particle[i]->boba = (float)Random()/2000.0f;
	
	particle[i]->rad = (float)Random()/1600.0f;
	
	particle[i]->pos.x = particle[i]->rad * sin(degToRad * particle[i]->yr);
	particle[i]->pos.z = particle[i]->rad * cos(degToRad * particle[i]->yr);
	
	particle[i]->gravity.y = -3.0f;
	particle[i]->child_maxlife = (float)(Random()+65000)/200000.0f;
	
	for (j=0;j<MAX_CHILDREN;j++)
	{
		particle[i]->child[j].pos.x = particle[i]->pos.x;
		particle[i]->child[j].pos.y = particle[i]->pos.y;
		particle[i]->child[j].pos.z = particle[i]->pos.z;
		particle[i]->child[j].dead = false;
		particle[i]->child[j].life = j*particle[i]->child_maxlife/MAX_CHILDREN;
	}
}

- (void)CreateRandomFirework:(int) i
{
	int j;
	
	particle[i] = malloc(sizeof(particles));
	
	particle[i]->r= (float)(Random()+32000)/65000.0f;
	particle[i]->g= (float)(Random()+32000)/65000.0f;
	particle[i]->b= (float)(Random()+32000)/65000.0f;
	
	particle[i]->spawnt = Time();
	
	particle[i]->pos.x = 0.0f;
	particle[i]->pos.z = 0.0f;
	particle[i]->pos.y = 0.0f;
	
	particle[i]->vel.x = (float)(Random())/33000.0f;
	particle[i]->vel.z = (float)(Random())/33000.0f;
	particle[i]->vel.y = (float)(Random()+32000)/6000.0f;
	
	particle[i]->gravity.y = -3.0f;
	particle[i]->child_maxlife = (float)(Random()+65000)/20000.0f;
	
	for (j=0;j<MAX_CHILDREN;j++)
	{
		particle[i]->child[j].pos.x = particle[i]->pos.x;
		particle[i]->child[j].pos.y = particle[i]->pos.y;
		particle[i]->child[j].pos.z = particle[i]->pos.z;
		particle[i]->child[j].dead = true;
		particle[i]->child[j].life = j*particle[i]->child_maxlife/MAX_CHILDREN;
	}
}

@end
