#import "Sier3d.h"

#import "sierpinski3d.c"

@implementation Sier3d	

- (id)initWithFrame:(NSRect)frameRect isPreview:(BOOL)preview
{
    gs = (gasketstruct *)malloc(sizeof(gasketstruct));
    
    self = [super initWithFrame:frameRect isPreview:preview];
        
    if (self) {
	NSOpenGLPixelFormatAttribute attribs[] =
        {	NSOpenGLPFAAccelerated,
		NSOpenGLPFADepthSize, 16,
		NSOpenGLPFAMinimumPolicy,
		NSOpenGLPFAClosestPolicy,
                0
        };
	
	NSOpenGLPixelFormat *format = [[[NSOpenGLPixelFormat alloc] initWithAttributes:attribs] autorelease];
		
        _view = [[[NSOpenGLView alloc] initWithFrame:NSZeroRect pixelFormat:format] autorelease];
        [self addSubview:_view];
    }
    
    speed=200;
    max_depth=6;
    
    timer_=NULL;
    
    #if DEBUG
    frameCount=0;
    #endif
    
    return self;
}

- (void)dealloc
{
    if (glIsList(gs->gasket1)) glDeleteLists(gs->gasket1, 1);
    free(gs);
    [_view removeFromSuperview];
    
    if (timer_!=NULL)
    {
        [timer_ invalidate];
        [timer_ release];
        timer_=NULL;
    }

    [super dealloc];
}

- (void)setFrameSize:(NSSize)newSize
{
    [super setFrameSize:newSize];
    [_view setFrameSize:newSize];
    _initedGL = NO;

}


- (void)animateOneFrame
{
    [[_view openGLContext] makeCurrentContext];
        if (!_initedGL) {
    	gs->view_rotx = SSRandomFloatBetween(0,360);
	gs->view_roty = SSRandomFloatBetween(0,360);
	gs->view_rotz = SSRandomFloatBetween(0,360);
	gs->angle = SSRandomFloatBetween(0,360);

        reshape([self frame].size.width, [self frame].size.height);

        pinit(gs);
        draw_gasket(gs);

        _initedGL = YES;
	compute_faceNormals();
    }

    draw_gasket(gs);    
    
    return;
}


- (BOOL)hasConfigureSheet { return NO; }
- (NSWindow*)configureSheet { return nil; }

- (void)stopAnimation
{
    if (timer_!=NULL)
    {
        [timer_ invalidate];
        [timer_ release];
        timer_=NULL;
    }
    
    _initedGL = NO;
    
    [super stopAnimation];
}

- (void) frameLog:(id) inSender
{
    fprintf(stderr,"%f frames/second\n",frameCount/10.0);
    frameCount=0;
}



@end
